package flashbuy

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/xorm"
)

// FlashbuyDAO 当前数据库操作对象
type FlashbuyDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewFlashbuyDAO New Inst
func NewFlashbuyDAO(ctx *utils.Context) *FlashbuyDAO {
	return &FlashbuyDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

type FlashBuy struct {
	model.TaFlashBuy `xorm:"extends"`
	CaseName         string
}
type CustomerFlashBuy struct {
	model.TaCustomerFlashBuy `xorm:"extends"`
	CaseName                 string
	CustomerName             string
	Phone                    string
}

func (m *FlashbuyDAO) GetFlashBuyList(caseid, flashBuyName, flashBuyStatus string, page, pageSize int) ([]FlashBuy, error) {
	var flashBuy []FlashBuy
	sql := `SELECT
	a.*,
	b.case_name 
FROM
	ta_flash_buy a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	Where a.status > ` + strconv.Itoa(models.STATUS_DEL) + `
	and a.case_id in(` + strings.Replace(caseid, ",", "','", -1) + `')`
	if flashBuyName != "" {
		sql += ` and a.flash_buy_name = '` + flashBuyName + `' `
	}
	if flashBuyStatus != "" {
		sql += ` and a.flash_buy_status ='` + flashBuyStatus + `'`
	}
	sql += ` order by a.start_date desc limit ` + strconv.Itoa(page-1) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&flashBuy)
	return flashBuy, err
}

func (m *FlashbuyDAO) GetFlashBuyListCount(caseid, flashBuyName, flashBuyStatus string) (int, error) {
	var flashBuy []model.TaFlashBuy
	sql := `SELECT
	a.*
FROM
	ta_flash_buy a
	Where a.status > ` + strconv.Itoa(models.STATUS_DEL) + `
	and a.case_id in(` + strings.Replace(caseid, ",", "','", -1) + `')`
	if flashBuyName != "" {
		sql += ` and a.flash_buy_name = '` + flashBuyName + `' `
	}
	if flashBuyStatus != "" {
		sql += ` and a.flash_buy_status ='` + flashBuyStatus + `'`
	}
	err := m.db.Sql(sql).Find(&flashBuy)
	return len(flashBuy), err
}

func (m *FlashbuyDAO) GetFlashBuyById(flashBuyId string) (*model.TaFlashBuy, error) {
	var flashBuy []model.TaFlashBuy
	err := m.db.Where("flash_buy_id = ?", flashBuyId).And("status > ?", models.STATUS_DEL).Find(&flashBuy)
	if err != nil {
		return nil, err
	}
	if len(flashBuy) > 0 {
		return &flashBuy[0], nil
	}
	return nil, nil
}

func (m *FlashbuyDAO) AddNewFlashBuy(flashbuy model.TaFlashBuy) (*model.TaFlashBuy, error) {
	flashbuy.FlashBuyId = utils.GetGUID()
	flashbuy.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(flashbuy)
	return &flashbuy, err
}

func (m *FlashbuyDAO) DeleteFlashBuy(flashBuyId string) error {
	var flashBuy = model.TaFlashBuy{
		FlashBuyId: flashBuyId,
		Status:     models.STATUS_DEL,
	}
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("flash_buy_id = ?", flashBuy.FlashBuyId).Update(flashBuy)
	return err
}

func (m *FlashbuyDAO) UpdateFlashBuy(flashBuyId, flashBuyStatus string) error {
	var flashBuy = model.TaFlashBuy{
		FlashBuyId:     flashBuyId,
		FlashBuyStatus: flashBuyStatus,
	}
	var cols = []string{
		"flash_buy_status",
	}
	_, err := m.db.Cols(cols...).Where("flash_buy_id =?", flashBuy.FlashBuyId).Update(flashBuy)
	return err
}

func (m *FlashbuyDAO) EditFlashBuy(flashBuy model.TaFlashBuy) error {
	var cols = []string{
		"flash_buy_name",
		"flash_buy_info",
	}
	_, err := m.db.Cols(cols...).Where("flash_buy_id = ?", flashBuy.FlashBuyId).Update(flashBuy)
	return err
}
func (m *FlashbuyDAO) VerifyFlashBuy(customerFlashBuyId string) error {
	var customerFlashBuy = model.TaCustomerFlashBuy{
		CustomerFlashBuyId: customerFlashBuyId,
		VerifyStatus:       models.VERIFY_USED,
		VerifyDate:         time.Now(),
	}
	var cols = []string{
		"verify_status",
		"verify_date",
	}
	_, err := m.db.Cols(cols...).Where("customer_flash_buy_id = ?", customerFlashBuy.CustomerFlashBuyId).Update(customerFlashBuy)
	return err
}

func (m *FlashbuyDAO) GetCustomerFlashBuyById(flashBuyId, phone string, page, pageSize int) ([]CustomerFlashBuy, error) {
	var customerFlashBuy []CustomerFlashBuy
	sql := `SELECT
	a.*,
	b.customer_name,
	b.phone,
	c.case_name 
FROM
	ta_customer_flash_buy a
	INNER JOIN ta_customer b ON a.customer_id = b.customer_id
	INNER JOIN sys_case c ON a.case_id = c.case_id 
WHERE
	a.flash_buy_id = '` + flashBuyId + `' 
	and a.status >` + strconv.Itoa(models.STATUS_DEL)
	if phone != "" {
		sql += ` and b.phone = '` + phone + `'`
	}
	sql += ` order by a.create_date desc  limit ` + strconv.Itoa(page-1) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&customerFlashBuy)
	return customerFlashBuy, err

}
func (m *FlashbuyDAO) GetCustomerFlashBuyByIdCount(flashBuyId, phone string) (int, error) {
	var customerFlashBuy []CustomerFlashBuy
	sql := `SELECT
	a.*,
	b.customer_name,
	b.phone,
	c.case_name 
FROM
	ta_customer_flash_buy a
	INNER JOIN ta_customer b ON a.customer_id = b.customer_id
	INNER JOIN sys_case c ON a.case_id = c.case_id 
WHERE
	a.flash_buy_id = '` + flashBuyId + `' 
	and a.status >` + strconv.Itoa(models.STATUS_DEL)
	if phone != "" {
		sql += ` and b.phone = '` + phone + `'`
	}
	err := m.db.Sql(sql).Find(&customerFlashBuy)
	return len(customerFlashBuy), err
}

func (m *FlashbuyDAO) GetCustomerFlashBuyByQr(customerFlashBuyId string) (*CustomerFlashBuy, error) {
	var customerFlashBuy []CustomerFlashBuy
	sql := `SELECT
	a.*,
	b.customer_name,
	b.phone,
	c.case_name 
FROM
	ta_customer_flash_buy a
	INNER JOIN ta_customer b ON a.customer_id = b.customer_id
	INNER JOIN sys_case c ON a.case_id = c.case_id 
WHERE
	a.customer_flash_buy_id ='` + customerFlashBuyId + `'`
	sql += ` and a.validate_start <= now() and a.validate_end >= now()`
	err := m.db.Sql(sql).Find(&customerFlashBuy)
	if len(customerFlashBuy) > 0 {
		return &customerFlashBuy[0], err
	}
	return nil, nil
}

// Update

// FlashBuy 抢购
// func (m *FlashbuyDAO) FlashBuy() error {

// }
