package flashbuy

import (
	"errors"
	"spaceofcheng/services/models/flashbuy"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/verify"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// FlashBuyServ 系统处理
type FlashBuyServ struct {
	ctx  *utils.Context
	dao  *flashbuy.FlashbuyDAO
	vdao *verify.VerifyDAO
}

// NewFlashBuyServ 初始化
func NewFlashBuyServ(ctx *utils.Context) *FlashBuyServ {
	return &FlashBuyServ{
		ctx:  ctx,
		dao:  flashbuy.NewFlashbuyDAO(ctx),
		vdao: verify.NewVerifyDAO(ctx),
	}
}

func (s *FlashBuyServ) GetFlashBuyList(caseid, flashBuyName, flashBuyStatus string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	flashbuys, err := s.dao.GetFlashBuyList(caseid, flashBuyName, flashBuyStatus, page, pageSize)
	if err != nil {
		utils.LogError("获取抢购活动列表失败: " + err.Error())
		return nil, errors.New("获取抢购活动列表失败")
	}
	total, err := s.dao.GetFlashBuyListCount(caseid, flashBuyName, flashBuyStatus)
	if err != nil {
		utils.LogError("获取抢购活动列表失败: " + err.Error())
		return nil, errors.New("获取抢购活动列表失败")
	}
	return map[string]interface{}{
		"list":     flashbuys,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}
func (s *FlashBuyServ) GetFlashBuyById(flashBuyId string) (*model.TaFlashBuy, error) {
	flashbuy, err := s.dao.GetFlashBuyById(flashBuyId)
	if err != nil {
		utils.LogError("获取抢购活动信息失败: " + err.Error())
		return nil, errors.New("获取抢购活动信息失败")
	}
	return flashbuy, nil
}
func (s *FlashBuyServ) SaveFlashBuy(flashbuy model.TaFlashBuy) (*model.TaFlashBuy, error) {
	var newFlashBuy *model.TaFlashBuy
	var err error
	if flashbuy.FlashBuyId == "" {
		newFlashBuy, err = s.dao.AddNewFlashBuy(flashbuy)
	} else {
		err = s.dao.EditFlashBuy(flashbuy)
		newFlashBuy = &flashbuy
	}
	if err != nil {
		utils.LogError("保存抢购活动信息失败: " + err.Error())
		return nil, errors.New("保存抢购活动信息失败")
	}
	return newFlashBuy, nil
}

func (s *FlashBuyServ) UpdateFlashBuy(flashBuyId, flashBuyStatus string) error {
	err := s.dao.UpdateFlashBuy(flashBuyId, flashBuyStatus)
	if err != nil {
		utils.LogError("更新活动状态失败: " + err.Error())
		return errors.New("更新活动状态失败")
	}
	return err
}
func (s *FlashBuyServ) DeleteFlashBuy(flashBuyId string) error {
	err := s.dao.DeleteFlashBuy(flashBuyId)
	if err != nil {
		utils.LogError("删除抢购活动信息失败: " + err.Error())
		return errors.New("删除抢购活动信息失败")
	}
	return err
}

func (s *FlashBuyServ) GetCustomerFlashBuyById(flashBuyId, phone string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	flashbuys, err := s.dao.GetCustomerFlashBuyById(flashBuyId, phone, page, pageSize)
	if err != nil {
		utils.LogError("获取记录列表失败: " + err.Error())
		return nil, errors.New("获取获取记录列表失败")
	}
	total, err := s.dao.GetCustomerFlashBuyByIdCount(flashBuyId, phone)
	if err != nil {
		utils.LogError("获取记录列表失败: " + err.Error())
		return nil, errors.New("获取获取记录列表失败")
	}
	return map[string]interface{}{
		"list":     flashbuys,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

func (s *FlashBuyServ) VerifyCustomerFlashBuyList(qrcode string) (*flashbuy.CustomerFlashBuy, error) {
	var err error
	var customerFlashBuyId string
	var customerVerify *flashbuy.CustomerFlashBuy
	customerFlashBuyId, err = s.vdao.GetCustomerCourseIdByQrcode(qrcode)
	if err != nil {
		utils.LogError("获取核销列表失败: " + err.Error())
		return nil, errors.New("获取核销列表失败")
	}
	customerVerify, err = s.dao.GetCustomerFlashBuyByQr(customerFlashBuyId)
	if err != nil {
		utils.LogError("获取核销列表失败: " + err.Error())
		return nil, errors.New("获取核销列表失败")
	}
	return customerVerify, nil

}

func (s *FlashBuyServ) Verify(customerFlashBuyId string) error {
	err := s.dao.VerifyFlashBuy(customerFlashBuyId)
	if err != nil {
		utils.LogError("核销失败: " + err.Error())
		return errors.New("核销失败")
	}
	return nil
}
